'use strict';

exports.__esModule = true;
exports.fontFace = exports.inserted = exports.sheet = undefined;
exports.flush = flush;
exports.css = css;
exports.injectGlobal = injectGlobal;
exports.keyframes = keyframes;
exports.hydrate = hydrate;

var _sheet = require('./sheet');

var _hash = require('./hash');

var _hash2 = _interopRequireDefault(_hash);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var sheet = exports.sheet = new _sheet.StyleSheet();
sheet.inject();

var inserted = exports.inserted = {};

function values(cls, vars) {
  var hash = (0, _hash2.default)([cls].concat(vars));
  var varCls = 'vars-' + hash;
  if (inserted[hash]) {
    return varCls;
  }
  var src = vars.map(function (val, i) {
    return '--' + cls + '-' + i + ': ' + val;
  }).join('; ');
  sheet.insert('.' + varCls + ' {' + src + '}');
  inserted[hash] = true;

  return varCls;
}

function flush() {
  sheet.flush();
  exports.inserted = inserted = {};
  sheet.inject();
}

function css(classes, vars, content) {
  if (content) {
    // inline mode
    var src = content.apply(undefined, vars); // returns an array
    var hash = (0, _hash2.default)(src);

    if (!inserted[hash]) {
      inserted[hash] = true;
      src.map(function (r) {
        return r.replace(new RegExp(classes[0], 'gm'), classes[0] + '-' + hash);
      }).forEach(function (r) {
        return sheet.insert(r);
      });
    }
    return classes[0] + '-' + hash + ' ' + classes.join(' ');
  }
  return classes.join(' ') + (vars && vars.length > 0 ? ' ' + values(classes[0], vars) : '');
}

function injectGlobal(src) {
  var hash = (0, _hash2.default)(src);
  if (!inserted[hash]) {
    inserted[hash] = true;
    src.forEach(function (r) {
      return sheet.insert(r);
    });
  }
}

var fontFace = exports.fontFace = injectGlobal;

function keyframes(kfm, src) {
  var hash = (0, _hash2.default)(src);
  var animationName = kfm + '-' + hash;
  if (!inserted[hash]) {
    inserted[hash] = true;
    src.forEach(function (r) {
      sheet.insert('@keyframes ' + animationName + ' ' + r);
    });
  }
  return animationName;
}

function hydrate(ids) {
  ids.forEach(function (id) {
    return inserted[id] = true;
  });
}